local logbook = {}

local wm = WINDOW_MANAGER
local lb_frame
local lc = GetCVar("language.2")

function logbook_init(eventCode, addOnName)
	if (addOnName ~= "LogBook") then return end
	
	local defaults = {		
        entries = {},
    }	
	logbook = ZO_SavedVars:New("LogBookSavedVars", 2, nil, defaults)
	
	if ( logbook["count"] == nil ) then
		logbook["count"] = 1
	end
	buildBookInterface()
	ZO_CreateStringId("SI_BINDING_NAME_TOGGLELOGBOOK", "LogBook")
end

function buildBookInterface()
	--main window
	lb_frame = wm:CreateTopLevelWindow("LogBook")
	lb_frame:SetDimensions(800, 625)
	lb_frame:SetClampedToScreen(true)
	lb_frame:SetMouseEnabled(true)
	lb_frame:SetMovable(true)
	lb_frame:SetHandler("OnReceiveDrag", function(self)
		end)
	lb_frame:SetHandler("OnMouseUp", function(self)
			self:StopMovingOrResizing()
			local _,a,_,b,x,y = self:GetAnchor()
		end)
		
	--window bg
	lb_frame.bg = wm:CreateControl("LogBook_bg", lb_frame, CT_TEXTURE)
	lb_frame.bg:SetTexture("EsoUI\\Art\\Miscellaneous\\listitem_backdrop.dds")
	lb_frame.bg:SetTextureCoords(0, 0.98, 0, 0.85)
	lb_frame.bg:SetAnchorFill(lb_frame)
	
	--window title
	lb_frame.title = wm:CreateControl("LogBook_title", lb_frame, CT_LABEL)
	lb_frame.title:SetAnchor(TOPLEFT, lb_frame, TOPLEFT, 50, 50)
	lb_frame.title:SetFont("ZoFontWindowTitle")
	lb_frame.title:SetText("LogBook")
	
	--divider below title
	lb_frame.titleDiv = wm:CreateControl("LogBook_div", lb_frame, CT_TEXTURE)
	lb_frame.titleDiv:SetDimensions(800, 4)
	lb_frame.titleDiv:SetAnchor(TOPLEFT, lb_frame.title, BOTTOMLEFT, -50, 5)
	lb_frame.titleDiv:SetTexture("EsoUI\\Art\\Miscellaneous\\centerscreen_topDivider.dds")
		
	--editbox bg
	lb_frame.editbg = wm:CreateControlFromVirtual("LogBook_edit_bg", lb_frame, "ZO_EditBackdrop")
	lb_frame.editbg:SetDimensions(650, 300)
	lb_frame.editbg:SetAnchor(TOPLEFT, lb_frame.titleDiv, BOTTOMLEFT, 70, 30)
	lb_frame.editbg:SetAnchor(BOTTOMLEFT, lb_frame, BOTTOMLEFT, 70, -60)
	
	--editbox
	lb_frame.edit = wm:CreateControlFromVirtual("LogBook_edit", lb_frame.editbg, "ZO_DefaultEditMultiLineForBackdrop")
	lb_frame.edit:SetHandler("OnTab", function() if IsShiftKeyDown() then lb_frame.titleEdit:TakeFocus() end end)
	lb_frame.edit:SetHandler("OnEscape", lb_frame.edit.LoseFocus)
	lb_frame.edit:SetMaxInputChars(30000000)
	lb_frame.edit:SetEditEnabled(false)
	
	lb_frame:SetMovable(true)
	lb_frame:SetHidden(true)
		
end
	
function toggleLogBook()
	if ( lb_frame:IsHidden() == true ) then
		lb_frame.edit:Clear()
		for i = (logbook["count"] - 19), logbook["count"] do
			lb_frame.edit:InsertText(logbook.entries[i])
			lb_frame.edit:InsertText("\n")
			i = i + 1
		end		
		lb_frame:SetHidden(false)
	elseif ( lb_frame:IsHidden() == false ) then
		lb_frame:SetHidden(true)
	end	
	
end
	
function addEntry(value)
	logbook["count"] = logbook["count"] + 1
	if not ( value == nil ) or not ( value == "" ) then
		logbook.entries[logbook["count"]] = "[" .. GetDateStringFromTimestamp(GetTimeStamp()) .. ", " .. GetTimeString() .. "] " .. value
	end
end

-- Aktive Events ------------------------------------------------------------------------------------------

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_QUEST_ADDED, function(eventID, journalIndex, questName, objectiveName)
	if ( lc == "de" ) then
		addEntry("'" .. questName .. "'" .. " angenommen!")
	else
		addEntry("'" .. questName .. "'" .. " accepted!")
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_QUEST_COMPLETE_EXPERIENCE, function(eventID, quest)
	if ( lc == "de" ) then
		addEntry("'" .. quest .. "'" .. " abgeschlossen!")
	else
		addEntry("'" .. quest .. "'" .. " completed!")
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_PLAYER_DEAD, function(eventID)
	if ( lc == "de" ) then
		addEntry("Gestorben!")
	else
		addEntry("Died!")
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_LEVEL_UPDATE, function(eventID, level)
	if ( lc == "de" ) then
		addEntry("Stufe " .. level .. " erreicht!")
	else
		addEntry("Level " .. level .. " reached!")
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_PLAYER_COMBAT_STATE, function(eventID, inCombat)
	if (inCombat) then
		if ( lc == "de" ) then
			addEntry("In Kampf verwickelt!")
		else
			addEntry("Involved in combat!")
		end
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_BROADCAST, function(eventID, message)
	if ( lc == "de" ) then
		addEntry("Nachricht " .. message)
	else
		addEntry("Message " .. message)
	end
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_ZONE_CHANGED, function(eventID, zoneName, subZoneName, var3)

	local zone
	if not (zoneName == nil) or not (zoneName == "") then
		zone = zoneName
	end
	if not (subZoneName == nil) or not (subZoneName == "") then
		zone = zone .. subZoneName
	end
	if (zone == nil) or (zone == "") or (zone == " ") then
		return
	end

	if ( logbook["count"] == nil ) then -- Workaround für count == nil beim Login
		zo_callLater(	function()
				if ( lc == "de" ) then
					addEntry(zone .. " betreten!")
				else
					addEntry(zone .. " entered!")
				end
		end, 5000)
	else
		if ( lc == "de" ) then
			addEntry(zone .. " betreten!")
		else
			addEntry(zone .. " entered!")
		end
	end
	
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_ZONE_UPDATE, function(eventID, unitTag, zone)	
	if not (zone == nil) or not (zone == "") then
		if ( lc == "de" ) then
			addEntry(zone .. " betreten!")
		else
			addEntry(zone .. " entered!")
		end
	end
	d("LOGBOOK-DEBUGGER: EVENT_ZONE_UPDATE - Wert von unittag: " .. unitTag)
end)

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_ACHIEVEMENT_AWARDED, function(eventID, name)
	if not (name == nil) or not (name == "") then
		if ( lc == "de" ) then
			addEntry("Errungenschaft erhalten: " .. name)
		else
			addEntry("Got Achievement: " .. name)
		end
	end
	d("LOGBOOK-DEBUGGER: EVENT_ZONE_UPDATE - Wert von unittag: " .. unitTag)
end)


--[[ Events welche hinzugefügt werden könnten
EVENT_OPEN_BANK
EVENT_BANKED_MONEY_UPDATE (integer eventCode, integer newBankedMoney, integer oldBankedMoney) 
EVENT_BANK_IS_FULL
EVENT_VETERAN_RANK_UPDATE (string unitTag, integer rank) 
EVENT_MEDAL_AWARDED (string name, string texture, string condition) 
EVENT_INVENTORY_IS_FULL (integer numSlotsRequested, integer numSlotsFree) 
EVENT_INVENTORY_ITEM_DESTROYED (integer itemSoundCategory) 
EVENT_END_SOUL_GEM_RESURRECTION 
EVENT_GROUP_INVITE_RECEIVED (string inviterName) 
EVENT_SHOW_BOOK (string bookTitle, string body, integer medium, bool showTitle) 
EVENT_SIEGE_BUSY (string siegeName)
]]

-----------------------------------------------------------------------------------------------------------


--[[ Event-Template

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_NAME, function(eventID, var1, var2, var3)
	addEntry(var1)	
end)

]]	


--[[ Deaktivierte Events

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_COMBAT_EVENT , function(eventID, result, isError,  abilityName,  abilityGraphic,  abilityActionSlotType,  sourceName,  sourceType,  targetName,  targetType,  hitValue,  powerType,  damageType, log)
addEntry("Kampf mit " .. targetName .. " aufgenommen!")
 end)  

-- Titel wurde deaktiviert weil die Ausgabe wie folgt lautet: "Der Titel wurde auf player geändert!" 
EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_TITLE_UPDATE, function(eventID, unitTag)
	if ( logbook["count"] == nil ) then -- Workaround für count == nil beim Login
		zo_callLater(	function()
			addEntry("Der Titel wurde auf " .. unitTag .. " geändert!")
		end, 5000)
	else
		addEntry("Der Titel wurde auf " .. unitTag .. " geändert!")
	end
end)
 
 ]]

 
-- TEST / --------------------------------------------------------------------------------------------------

function testentry()

	local value = "Test-Eintrag"
	addEntry(value)
	d("Eintrag erstellt! Anzahl der Einträge: " .. logbook["count"])
	lb_frame:SetHidden(true)
	
end
SLASH_COMMANDS["/lb_test"] = testentry

function testshow()

	d("-------------------------------")
	d("Anzahl der Einträge: " .. logbook["count"])
	for i = 1, logbook["count"] do
		d(logbook[i])
		i = i + 1
	end
	d("-------------------------------")		
	
end
SLASH_COMMANDS["/lb_list"] = testshow

-- / TEST --------------------------------------------------------------------------------------------------

EVENT_MANAGER:RegisterForEvent("LogBook", EVENT_ADD_ON_LOADED, logbook_init)